@@ -13,3 +13,6 @@ from jyhelper.fileHelper import fileHelper
 from jyhelper.feishuRobot import feishuRobot
 from jyhelper.sysHelper import sysHelper
 from jyhelper.mysql import mysql
+from jyhelper.strHelper import strHelper
+from jyhelper.listHelper import listHelper
+from jyhelper.dictHelper import dictHelper
