@@ -41,7 +41,7 @@ class timeHelper:
     @staticmethod
     def yesterday():
         """返回昨天的日期"""
-        return timeHelper.addDay(None,-1)
+        return timeHelper.addDay(None, -1)
 
     # 获取时间戳
     @staticmethod
@@ -149,7 +149,7 @@ class timeHelper:
 
     # 得到一个日期列表(默认包含首尾)
     @staticmethod
-    def getDayList(startDate, endDate, stepLen=86400,includeTail=True):
+    def getDayList(startDate, endDate, stepLen=86400, includeTail=True):
         if timeHelper.checkIsDay(startDate) and timeHelper.checkIsDay(endDate):
             formatStr = '%Y-%m-%d'
         elif timeHelper.checkIsDate(startDate) and timeHelper.checkIsDate(endDate):
@@ -159,10 +159,34 @@ class timeHelper:
         timeStart = timeHelper.getTime(startDate)
         timeEnd = timeHelper.getTime(endDate)
         res = []
-        for i in range(timeStart,timeEnd+int(includeTail),stepLen):
-            res.append(timeHelper.getDay(i,formatStr))
+        for i in range(timeStart, timeEnd + int(includeTail), stepLen):
+            res.append(timeHelper.getDay(i, formatStr))
         return res
 
+    @staticmethod
+    def get_hour(date=None, year=False, month=False, day=False, minute=False, second=False):
+        """返回日期字符串中的小时部分 转换为整数返回"""
+        if date is None:
+            date = timeHelper.getDate()
+        i1 = 11
+        i2 = 13
+        if year:
+            i1 = 0
+            i2 = 4
+        if month:
+            i1 = 5
+            i2 = 7
+        if day:
+            i1 = 8
+            i2 = 10
+        if minute:
+            i1 = 14
+            i2 = 16
+        if second:
+            i1 = 17
+            i2 = 19
+        return int(date[i1:i2])
+
 
 if __name__ == '__main__':
     # day = '2023-11-12'
@@ -173,4 +197,9 @@ if __name__ == '__main__':
     # for row in timeHelper.getDayList('2024-01-01 00:00:00','2024-01-02 00:00:00',includeTail=False,stepLen=3600):
     #     print(row)
 
-    print(timeHelper.yesterday())
\ No newline at end of file
+    print(timeHelper.get_hour('2024-11-22 13:25:35', year=True))
+    print(timeHelper.get_hour('2024-11-22 13:25:35', month=True))
+    print(timeHelper.get_hour('2024-11-22 13:25:35', day=True))
+    print(timeHelper.get_hour('2024-11-22 13:25:35'))
+    print(timeHelper.get_hour('2024-11-22 13:25:35', minute=True))
+    print(timeHelper.get_hour('2024-11-22 13:25:35', second=True))
